﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Globalization;
using System.Reflection;
using System.Text;
using System.Transactions;
using Produmex.Foundation;
using Produmex.Foundation.Barcode;
using Produmex.Foundation.Data;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Convertors;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.GS1;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.DataObjects;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.SlimScreen.WinGui;
using Produmex.Foundation.Utilities;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo;
using Produmex.Foundation.Wwf.Sbo.DataObjects;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Devices;
using Produmex.Sbo.Logex.Data.Generators.BatchNumberGenerators;
using Produmex.Sbo.Logex.Data.Generators.PackingController;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Data.ViewObjects.Definitions.Views;

// PmxWorkflowExecutionTypes.HOOK_FLOW
namespace Produmex.Sbo.Logex.WorkflowScripts
{
    public class WorkflowScript_PickingAfterItemToPickIdentifiedHookScript : WorkflowInstanceScriptBase
    {
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        // Input parameters * do not change *
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;

        public ReadOnlyBinder<PickingItemInfo> PickingItemInfo;

        // Output parameters * do not change *
        public ReadWriteBinder<bool> BackRequested;

        // Sub-flows
        // <none>

        public WorkflowScript_PickingAfterItemToPickIdentifiedHookScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {
        }

        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {
            // Parameters in scope
            Session session = GetScopeParameter("Session") as Session;
            ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;
            Message msg = null;
            PickingItemInfo itemInfo = PickingItemInfo;
           

            string query = BuildQuery.GetQueryGetBatchAttributeValue1(itemInfo.PickListDocEntry, itemInfo.PickListLineID);
            DataSet ds = sboProviderService.RunView(false, null, null, query);

            if (GetNrOfRows(ds) > 0)
            {
                string promoId = ds.Tables[0].Rows[0][Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables.PmxDocumentLineColumns.BatchAttributeValue1.NAME].ToString();

                query = BuildQuery.GetQueryOldestBatch(itemInfo.ItemCode, itemInfo.BestBeforeDate);
                ds = sboProviderService.RunView(false, null, null, query);
                if (GetNrOfRows(ds) > 0)
                {


                    if (!String.IsNullOrEmpty(promoId))
                    {
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            this.DefaultCultureInfo, BuildParamCollection(
                          //  "MessageKey", String.Format("Please pick promo {0}", promoId),
                          "MessageKey", "Pick appropriate BBE for this item",
                            "ShowButton", true
                            ));
                        msg = WaitForMessage();
                    }
                }
            }
        }

        #endregion

        private int GetNrOfRows(DataSet ds)
        {
            return (ds != null && ds.Tables.Count > 0) ? ds.Tables[0].Rows.Count : 0;
        }

        #region QUERY

        private class BuildQuery
        {
            public static string GetQueryGetBatchAttributeValue1(int pickListDocEntry, int pickListLineId)
            {
                StringBuilder strBuilder = new StringBuilder();

                strBuilder.AppendFormat("SELECT DISTINCT {0} \r\n FROM {1}",
                    SqlCommandHelper.EscapeTableColumnName(SboSalesOrderLineTable.NAME, Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables.PmxDocumentLineColumns.BatchAttributeValue1.NAME),
                    SqlCommandHelper.EscapeTableName(SboSalesOrderLineTable.NAME));

                strBuilder.AppendFormat("\r\n LEFT JOIN {0} ON {1} = {2} AND {3} = {4}",
                    SqlCommandHelper.EscapeTableName(PmxPickListProposalLineTable.NAME),
                    SqlCommandHelper.EscapeTableColumnName(SboSalesOrderLineTable.NAME, SboSalesOrderLineTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.BaseEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(SboSalesOrderLineTable.NAME, SboSalesOrderLineTable.Columns.LineNum.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.BaseLine.NAME));

                strBuilder.AppendFormat("\r\n LEFT JOIN {0} ON {1} = {2} AND {3} = {4}",
                    SqlCommandHelper.EscapeTableName(PmxPickListLineTable.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.BaseEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.LineNum.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.BaseLine.NAME));

                strBuilder.AppendFormat("\r\n WHERE {0} = {1} \r\n  AND {2} = {3}",
                    SqlCommandHelper.EscapeTableColumnName(PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.DocEntry.NAME),
                    pickListDocEntry.ToString(),
                    SqlCommandHelper.EscapeTableColumnName(PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.LineNum.NAME),
                    pickListLineId.ToString());

                return strBuilder.ToString();
            }

            public static string GetQueryOldestBatch(string itemCode, DateTime? bestBeforeDate)
            {


                StringBuilder strBuilder = new StringBuilder();
                strBuilder.AppendFormat("SELECT DISTINCT {0} \r \n FROM {1} ",
                    SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.InternalKey.NAME),
                    SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));

                strBuilder.AppendFormat("\r\n LEFT JOIN {0} ON {1} = {2}  ",
                SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME),
                SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME),
                SqlCommandHelper.EscapeTableColumnName(PmxItemTransactionalInfoTable.NAME, PmxInventoryTotalTable.Columns.InternalKey.NAME));

                strBuilder.AppendFormat("\r\n WHERE {0} = {1}",
                SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME),
                 SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));
               
                if (bestBeforeDate.HasValue)
                {
                    strBuilder.AppendFormat("\r\n AND  {0} < {1} ",
                    SqlCommandHelper.EscapeTableColumnName(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(SboHelper.ConvertDateToSboDate(bestBeforeDate.Value), CultureInfo.InvariantCulture));


                 
                }

                return strBuilder.ToString();
            }
        }



        #endregion
    }
}
