﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Globalization;
using System.Reflection;
using System.Text;
using Produmex.Foundation;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.Utilities;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Providers;

// PmxWorkflowExecutionTypes.HOOK_FLOW
namespace WorkflowScript_SalesDeliveryCreatedHookScript
{
    /// <summary>
    /// 
    /// </summary>
    public class WorkflowScript_SalesDeliveryCreatedHookScript : WorkflowInstanceScriptBase
    {
        private static readonly ILog s_log = LogProvider.GetLogger( MethodInfo.GetCurrentMethod().DeclaringType );


        // Input parameters * do not change *
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;
        public ReadOnlyBinder<DocumentRef[]> DocumentRefs;

        public static class Definitions
        {
            public const int REPORT_KEY = 13;

            public const string PRINT_DOCK = "DOCK";


        }

        // Output parameters
        // <none>

        // Sub-flows
        // <none>


        /// <summary>
        /// Initializes a new instance of the <see cref="WorkflowScript_SalesDeliveryCreatedHookScript"/> class.
        /// </summary>
        /// <param name="parent">The parent.</param>
        /// <param name="factory">The factory.</param>
        public WorkflowScript_SalesDeliveryCreatedHookScript( WorkflowInstanceBase parent, WorkflowInstanceFactory factory )
            : base( parent, factory )
        {
        }

        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {
            // Parameters in scope
            Session session = GetScopeParameter( "Session" ) as Session;
            ISboProviderService sboProviderService = GetScopeParameter( "<WwfService>ISboService" ) as ISboProviderService;

            Message msg = null;

            string query = null;
 
        Step_PrintLabels:            
            
            
            
            ProdumexError pmxError = null;
       

            string msgKey = null;

            DocumentRef[] documentRefs = DocumentRefs.Get();

            Collection<int> listOfDeliveries = new Collection<int>();

            foreach ( DocumentRef docRef in documentRefs )
            {
                if ( docRef.DocType == SboObjectTypes.SalesDelivery )
                {
                    listOfDeliveries.Add(docRef.DocEntry);
                }
            }

            if ( listOfDeliveries.Count > 0 )
            {
                try
                {
                    query = BuildQuery.GetDeliveryOrder( listOfDeliveries, sboProviderService.GetDbTool().CultureInfo );

                    DataSet ds = sboProviderService.RunView( false, null, null, query );

                    if (this.GetNumberOfRows(ds) > 0)
                    {
                        for (int i = 0; i < ds.Tables[0].Rows.Count; i++)
                        {
                            int deliveryDocEntry = Convert.ToInt32(ds.Tables[0].Rows[i][SboSalesDeliveryNoteTable.Columns.DocEntry.NAME]);

                            //Print label
                            s_log.InfoFormat("Route: Print the report for delivery {0}", deliveryDocEntry);

                            //print label
                            sboProviderService.InvokeMethodWithDbConnection<PmxOsePrinter>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOsePrinter>(ExecutePrintLabel), deliveryDocEntry, session.DeviceInstanceId, Definitions.PRINT_DOCK, 1);
							sboProviderService.InvokeMethodWithDbConnection<PmxOsePrinter>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOsePrinter>(ExecutePrintLabel), deliveryDocEntry, session.DeviceInstanceId, Definitions.PRINT_DOCK, 1);
						}
                    }
                    else
                    {
						foreach (int deliveryDocEntry in listOfDeliveries)
						{
							//Print label
							s_log.InfoFormat("NO route: Print the report for delivery {0}", deliveryDocEntry);

							//print label
							sboProviderService.InvokeMethodWithDbConnection<PmxOsePrinter>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOsePrinter>(ExecutePrintLabel), deliveryDocEntry, session.DeviceInstanceId, Definitions.PRINT_DOCK, 1);

						}
                    }

                }
                catch ( ProdumexException ex )
                {
                    s_log.Error( "", ex );
                    msgKey = ExceptionHelper.Format( ex, this.DefaultCultureInfo );
                    pmxError = ex.Error;
                }
                catch ( Exception ex )
                {
                    s_log.Error( "", ex );
                    msgKey = ExceptionHelper.Format( ex, this.DefaultCultureInfo );
                }

                if ( msgKey != null )
                {
                    session.ShowScreen( typeof( Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen ),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "InitialErrorKey", null,
                            "MessageKey", msgKey,
                            "PmxError", pmxError,
                            "ShowButton", true ) );
                    WaitForMessage();
                }

                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "MessageKey", "MSG_QUESTION_REPRINT_DOCUMENT",
                                "TitleKey", "MSG_TITLE_REPRINT_DOCUMENT"
                                ));

                msg = WaitForMessage();

                if (msg.Name.EndsWith(".Yes"))
                {
                    goto Step_PrintLabels;
                }
            }

        }

        #endregion



        #region HELPERS




        /// <summary>
        /// Executes the print label.
        /// </summary>
        /// <param name="conn">The connection.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxOsePrinter ExecutePrintLabel( PmxDbConnection conn, object[] parameters )
        {
            PmxPrintReportProvider prov = new PmxPrintReportProvider( conn );

            Collection<ReportParameter> reportParameters = new Collection<ReportParameter>();
            //reportParameters.Add( new ReportParameter( Produmex.Sbo.Logex.Data.Reports.Definitions.ReportTypeSalesDeliveryNoteDefinition.Parameters.SalesDeliveryNoteDocEntry.NAME, (int)parameters[0] ) );

            reportParameters.Add(new ReportParameter(Produmex.Sbo.Logex.Data.Reports.Definitions.ReportTypeSAPCrystalReportsDefinition.Parameters.DocEntry.NAME, (int)parameters[0]));
            reportParameters.Add(new ReportParameter(Produmex.Sbo.Logex.Data.Reports.Definitions.ReportTypeSAPCrystalReportsDefinition.Parameters.ObjectID.NAME, (int)SboObjectType.SalesDeliveryNote));


            return prov.PrintReport( reportParameters, Definitions.REPORT_KEY, (int) parameters[3], (string)parameters[1], (string)parameters[2], false );
        }


        /// <summary>
        /// Gets the number of rows.
        /// </summary>
        /// <param name="ds">The ds.</param>
        /// <returns></returns>
        private int GetNumberOfRows( DataSet ds )
        {
            if ( ds != null && ds.Tables.Count > 0 )
            {
                return ds.Tables[0].Rows.Count;
            }
            return 0;
        }

        #endregion

        #region QUERY

        private class BuildQuery
        {


            /// <summary>
            /// Gets the delivery order.
            /// </summary>
            /// <param name="docEntry">The document entry.</param>
            /// <param name="cultInfo">The cult information.</param>
            /// <returns></returns>
            internal static string GetDeliveryOrder( Collection<int> listOfDeliveries, CultureInfo cultInfo )
            {
                /*
                 select distinct odln.DocEntry,PMX_RTLI.sequence from ODLN
                 join DLN1 on ODLN.DocEntry = DLN1.DocEntry
                 join PMX_DLPL on PMX_DLPL.SalesDeliveryNoteLineDocEntry = DLN1.DocEntry
                 join PMX_PLLI on PMX_DLPL.PickListLineDocEntry = PMX_PLLI.DocEntry
                 join PMX_PLPH on PMX_PLLI.BaseType = 'PMX_PLPH' and pmx_plli.BaseEntry = PMX_PLPH.DocEntry
                 join PMX_RTLI on PMX_PLPH.RouteLineDocEntry = PMX_RTLI.DocEntry and PMX_plph.RouteLineLineNum = pmx_rtli.LineNum
                 where odln.DocEntry IN (LIST OF DOCENTRIES)
                 order by pmx_rtli.sequence desc 
                 */
                StringBuilder strBuilder = new StringBuilder();

                strBuilder.AppendFormat( "SELECT DISTINCT {0}",
                    SqlCommandHelper.EscapeTableColumnName( SboSalesDeliveryNoteTable.NAME, SboSalesDeliveryNoteTable.Columns.DocEntry.NAME ) );

                strBuilder.AppendFormat( ", {0}",
                    SqlCommandHelper.EscapeTableColumnName( PmxRouteLineTable.NAME, PmxRouteLineTable.Columns.Sequence.NAME ) );

                strBuilder.AppendFormat( "\r\nFROM {0}",
                    SqlCommandHelper.EscapeTableName( SboSalesDeliveryNoteTable.NAME ) );

                strBuilder.AppendFormat( "\r\nINNER JOIN {0} ON {1} = {2}",
                    SqlCommandHelper.EscapeTableName(SboSalesDeliveryNoteLineTable.NAME),
                    SqlCommandHelper.EscapeTableColumnName( SboSalesDeliveryNoteTable.NAME, SboSalesDeliveryNoteTable.Columns.DocEntry.NAME ),
                    SqlCommandHelper.EscapeTableColumnName( SboSalesDeliveryNoteLineTable.NAME, SboSalesDeliveryNoteLineTable.Columns.DocEntry.NAME ) );

                strBuilder.AppendFormat("\r\nINNER JOIN {0} ON {1} = {2} AND {3} = {4}",
                   SqlCommandHelper.EscapeTableName( PmxLinkSalesDeliveryNoteLinePickListLineTable.NAME ),
                   SqlCommandHelper.EscapeTableColumnName( PmxLinkSalesDeliveryNoteLinePickListLineTable.NAME, PmxLinkSalesDeliveryNoteLinePickListLineTable.Columns.SalesDeliveryNoteLineDocEntry.NAME ),
                   SqlCommandHelper.EscapeTableColumnName( SboSalesDeliveryNoteLineTable.NAME, SboSalesDeliveryNoteLineTable.Columns.DocEntry.NAME ),
                   SqlCommandHelper.EscapeTableColumnName( PmxLinkSalesDeliveryNoteLinePickListLineTable.NAME, PmxLinkSalesDeliveryNoteLinePickListLineTable.Columns.SalesDeliveryNoteLineLineNum.NAME ),
                   SqlCommandHelper.EscapeTableColumnName( SboSalesDeliveryNoteLineTable.NAME, SboSalesDeliveryNoteLineTable.Columns.LineNum.NAME ) );

                strBuilder.AppendFormat( "\r\nINNER JOIN {0} ON {1} = {2}",
                    SqlCommandHelper.EscapeTableName( PmxPickListLineTable.NAME ),
                    SqlCommandHelper.EscapeTableColumnName( PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.DocEntry.NAME ),
                    SqlCommandHelper.EscapeTableColumnName( PmxLinkSalesDeliveryNoteLinePickListLineTable.NAME, PmxLinkSalesDeliveryNoteLinePickListLineTable.Columns.PickListLineDocEntry.NAME ) );

                strBuilder.AppendFormat( "\r\nINNER JOIN {0} ON {1} = {2} AND {3} = {4}",
                   SqlCommandHelper.EscapeTableName( PmxPickListProposalTable.NAME ),
                   SqlCommandHelper.EscapeTableColumnName( PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.BaseType.NAME ),
                   SqlCommandHelper.FormatAndEscapeColumnValue(Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.PmxObjectTypes.PmxPickListProposal,cultInfo),
                    SqlCommandHelper.EscapeTableColumnName( PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.BaseEntry.NAME ),
                   SqlCommandHelper.EscapeTableColumnName( PmxPickListProposalTable.NAME, PmxPickListProposalTable.Columns.DocEntry.NAME ) );

                strBuilder.AppendFormat( "\r\nINNER JOIN {0} ON {1} = {2} AND {3} = {4}",
                   SqlCommandHelper.EscapeTableName( PmxRouteLineTable.NAME ),
                   SqlCommandHelper.EscapeTableColumnName( PmxPickListProposalTable.NAME, PmxPickListProposalTable.Columns.RouteLineDocEntry.NAME ),
                   SqlCommandHelper.EscapeTableColumnName( PmxRouteLineTable.NAME, PmxRouteLineTable.Columns.DocEntry.NAME ),
                    SqlCommandHelper.EscapeTableColumnName( PmxPickListProposalTable.NAME, PmxPickListProposalTable.Columns.RouteLineLineNum.NAME ),
                   SqlCommandHelper.EscapeTableColumnName( PmxRouteLineTable.NAME, PmxRouteLineTable.Columns.LineNum.NAME ) );

                strBuilder.AppendFormat( "\r\nWHERE {0} IN ({1})",
                    SqlCommandHelper.EscapeTableColumnName( SboSalesDeliveryNoteTable.NAME, SboSalesDeliveryNoteTable.Columns.DocEntry.NAME ),
                    SqlCommandHelper.FormatAndEscapeColumnValue( listOfDeliveries, cultInfo ) );

                strBuilder.AppendFormat( "\r\nORDER BY {0} DESC",
                   SqlCommandHelper.EscapeTableColumnName( PmxRouteLineTable.NAME, PmxRouteLineTable.Columns.Sequence.NAME ) );
               

                return strBuilder.ToString();
            }


        }

        #endregion

    }
}
